# -*-coding:utf8-*-

import json
import numpy as np
from matplotlib import pyplot as plt


def softmax(x):
    """Compute softmax values for each sets of scores in x."""
    e_x = np.exp(x - np.max(x))
    return e_x / e_x.sum()


def MPQAProcessing(lexicon, filepath):
    lines = map(lambda x: x.split(' '), open(filepath).readlines())
    original_lexicon_size = len(lexicon)
    for i in range(len(lines)):
        prior_polarity = lines[i][5].split('=')[1].strip('\n')
        word = lines[i][2].split('=')[1]
        if prior_polarity == "negative" or prior_polarity == "positive":
            if word.lower() not in lexicon:
                lexicon[word.lower()] = (1, 1.0)
            else:
                value_tmp = lexicon[word.lower()]
                lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1] + 1.0) / (value_tmp[0] + 1))
        else:
            if word.lower() not in lexicon:
                lexicon[word.lower()] = (1, 0.0)
            else:
                value_tmp = lexicon[word.lower()]
                lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1]) / (value_tmp[0] + 1))

    del lines
    print("The MPQA lexicon size is %d. The total lexicon size is %d." %
          (len(lexicon) - original_lexicon_size, len(lexicon)))


def SentiWordNet(lexicon, filepath):
    lines = map(lambda x: x.split('\t'), open(filepath).readlines())
    original_lexicon_size = len(lexicon)
    for i in range(len(lines)):
        pos_index = float(lines[i][2])
        neg_index = float(lines[i][3])
        senti_degree = pos_index + neg_index
        words = lines[i][4].split(' ')
        for j in range(len(words)):
            word = words[j].split('#')[0]
            if word.lower() not in lexicon:
                lexicon[word.lower()] = (1, senti_degree)
            else:
                value_tmp = lexicon[word.lower()]
                lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1] + senti_degree) / (value_tmp[0] + 1))
    del lines
    print("The SentiWordNet lexicon size is %d. The total lexicon size is %d." %
          (len(lexicon) - original_lexicon_size, len(lexicon)))


def SocialSent(lexicon, filepath):
    lines = map(lambda x: x.split('\t'), open(filepath).readlines())
    original_lexicon_size = len(lexicon)
    for i in range(len(lines)):
        pos_neg = abs(float(lines[i][1])) / 4.0
        if pos_neg > 1.0:
            pos_neg = 1.0
        word = lines[i][0]
        if word.lower() not in lexicon:
            lexicon[word.lower()] = (1, pos_neg)
        else:
            value_tmp = lexicon[word.lower()]
            lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1] + pos_neg) / (value_tmp[0] + 1))

    del lines
    print("The SocialSent lexicon size is %d. The total lexicon size is %d." %
          (len(lexicon) - original_lexicon_size, len(lexicon)))


def SST(lexicon, filepath):
    dictionary_path = filepath + "dictionary.txt"
    sentiment_labels_path = filepath + "sentiment_labels.txt"
    dictionary = dict()
    original_lexicon_size = len(lexicon)

    with open(dictionary_path) as f:
        for line in f:
            word = line.split('|')[0]
            id = line.split('|')[1].strip('\n')
            if len(word.split(' ')) == 1:
                dictionary[id] = word.lower()
    lines = map(lambda x: x.split('|'), open(sentiment_labels_path).readlines())
    for i in range(len(lines)):
        id = lines[i][0]
        score = abs(float(lines[i][1]) - 0.5) * 2.0
        if id in dictionary:
            word = dictionary[id]
            if word not in lexicon:
                lexicon[word.lower()] = (1, score)
            else:
                value_tmp = lexicon[word.lower()]
                lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1] + score) / (value_tmp[0] + 1))

    del dictionary
    print("The SST lexicon size is %d. The total lexicon size is %d." %
          (len(lexicon) - original_lexicon_size, len(lexicon)))


def ZhiWangSentiment(lexicon, filepath):
    original_lexicon_size = len(lexicon)
    lines = open(filepath).readlines()
    for i in range(len(lines)):
        if len(lines[i].split(' ')) == 2:
            word = lines[i].split(' ')[0]
            if word.lower() not in lexicon:
                lexicon[word.lower()] = (1, 1.0)
            else:
                value_tmp = lexicon[word.lower()]
                lexicon[word.lower()] = (value_tmp[0] + 1, (value_tmp[0] * value_tmp[1] + 1.0) / (value_tmp[0] + 1))
    del lines
    print("The ZhiWangSentiment lexicon size is %d. The total lexicon size is %d." %
          (len(lexicon) - original_lexicon_size, len(lexicon)))


def genSentimentLexicon():
    lexicon = dict()
    MPQAProcessing(lexicon, "/media/rowitzou/WORK/NLP/sentiment-classification/"
                            "data/sentiment lexicon/MPQA/lexicon.tff")
    SentiWordNet(lexicon, "/media/rowitzou/WORK/NLP/sentiment-classification/data/"
                          "sentiment lexicon/SentiWordNet_3.0.0/SentiWordNet_3.0.0_20130122.txt")
    # SocialSent(lexicon, "/media/rowitzou/WORK/NLP/sentiment-classification/"
    #                     "data/sentiment lexicon/socialsent/movies.tsv")
    SST(lexicon, "/media/rowitzou/WORK/NLP/sentiment-classification/data/"
                 "sentiment lexicon/stanfordSentimentTreebank/")
    ZhiWangSentiment(lexicon, "/media/rowitzou/WORK/NLP/sentiment-classification/data/"
                              "sentiment lexicon/zhiwang-sentiment/sentiment.txt")
    # lexicon = [(v[0], v[1][1]) for v in sorted(lexicon.items(), key=(lambda d: d[1][1]), reverse=True)]

    filename = open("./data/IMDB/Attention_Lexicon.json", "wb")
    json.dump(lexicon, filename)


genSentimentLexicon()
